#!/usr/bin/env python
# -*- coding: utf-8 -*-

# run tests with: python -m unittest test_data_load
#             or  python -m unittest discover

import os
import sys
import random
import tempfile
import gzip
import unittest
import fityk

#locale.setlocale(locale.LC_NUMERIC, 'C')

class FileLoadBase(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        prefix = getattr(self, 'file_prefix', 'tmp')
        f = tempfile.NamedTemporaryFile(prefix=prefix, delete=False)
        self.data = self.generate_data()
        for d in self.data:
            f.write(self.line_format(d))
        f.close()
        self.data.sort()
        self.filename = f.name

    def generate_data(self):
        return [(random.uniform(-100, 100), random.gauss(10, 20))
                for _ in range(30)]

    def tearDown(self):
        #print "remove " + self.filename
        os.unlink(self.filename)

    def compare(self, out, ndigits):
        self.assertEqual([i.x for i in out],
                         [round(i[0], ndigits) for i in self.data])
        self.assertEqual([i.y for i in out],
                         [round(i[1], ndigits) for i in self.data])

class TextFileLoadBase(FileLoadBase):
    def line_format(self, t):
        return " %.7f %.7f\n" % t
    def test_load(self):
        self.ftk.execute("@0 < '%s'" % self.filename)
        self.compare(self.ftk.get_data(), 7)
    def test_load_with_lua(self):
        self.ftk.execute("lua F:load([[%s]])" % self.filename)
        self.compare(self.ftk.get_data(), 7)
    def test_load_with_py(self):
        self.ftk.load(self.filename, 0)
        self.compare(self.ftk.get_data(), 7)

class TestText(TextFileLoadBase):
    def test_load_strict(self):
        self.ftk.execute("@0 < '%s' text strict" % self.filename)
        self.compare(self.ftk.get_data(), 7)
    def test_load_decimal_comma(self):
        # this option only replaces commas with dots, so it's still possible
        # to read file with decimal point '.'
        self.ftk.execute("@0 < '%s' text decimal_comma" % self.filename)
        self.compare(self.ftk.get_data(), 7)

class TestUtf8Filename(TextFileLoadBase):
    def setUp(self):
        self.file_prefix = 'tmp_Zażółć_gęślą_jaźń' # string, not u''
        FileLoadBase.setUp(self)

class TestFilenameWithQuotes(TextFileLoadBase):
    def setUp(self):
        self.file_prefix = """tmp'"!@#"""
        FileLoadBase.setUp(self)
    def test_load(self): pass # would fail

class TestTextComma(FileLoadBase):
    def line_format(self, t):
        return (" %.7f %.7f\n" % t).replace('.', ',')
    def test_load(self):
        self.ftk.execute("@0 < '%s' text decimal_comma" % self.filename)
        self.compare(self.ftk.get_data(), 7)
    def test_load_strict(self):
        self.ftk.execute("@0 < '%s' text decimal_comma strict" % self.filename)
        self.compare(self.ftk.get_data(), 7)
    def test_load_decimal_point(self): # should fail
        self.ftk.execute("@0 < '%s'" % self.filename)
        self.assertNotEqual([i.x for i in self.ftk.get_data()],
                            [round(i[0], 7) for i in self.data])


class Test5ColsFile(FileLoadBase):
    def generate_data(self):
        return [(random.uniform(-100, 100),
                 random.gauss(0, 20),
                 random.gauss(10, 1),
                 random.gauss(20, 1),
                 random.gauss(30, 1))
                for _ in range(30)]
    def line_format(self, t):
        return "%.7f %.7f %.7f %.7f %.7f\n" % t
    def test_load_default(self):
        self.ftk.execute("@0 < '%s'" % self.filename)
        self.compare(self.ftk.get_data(), ndigits=7)
        self.ftk.execute("@0 < '%s::::'" % self.filename)
        self.compare(self.ftk.get_data(), ndigits=7)
        self.ftk.execute("@0 < '%s:1:2::0'" % self.filename)
        self.compare(self.ftk.get_data(), ndigits=7)
    def test_load_with_index(self):
        self.ftk.execute("@0 < '%s:0:2::'" % self.filename)
        out = self.ftk.get_data()
        self.assertEqual([i.x for i in out], range(len(out)))
    def test_load_with_sigma(self):
        self.ftk.execute("@0 < '%s:1:2:3:'" % self.filename)
        out = self.ftk.get_data()
        self.assertEqual([i.sigma for i in out],
                         [round(i[2], 7) for i in self.data])
    def test_load_multi_y(self):
        self.ftk.execute("@+ < '%s:1:2..3,5:4:'" % self.filename)
        for n, y_col in enumerate([2,3,5]):
            out = self.ftk.get_data(n)
            self.assertEqual([i.x for i in out],
                             [round(i[0], 7) for i in self.data])
            self.assertEqual([i.y for i in out],
                             [round(i[y_col-1], 7) for i in self.data])
            self.assertEqual([i.sigma for i in out],
                             [round(i[3], 7) for i in self.data])


class TestSimpleScript(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        f = tempfile.NamedTemporaryFile(delete=False)
        f.write("M=1\n X[0]=1.1, Y[0]=-5, S[0]=0.8")
        f.close()
        self.filename = f.name

    def tearDown(self):
        os.unlink(self.filename)

    def test_simple_script(self):
        self.ftk.get_ui_api().exec_fityk_script(self.filename)
        data = self.ftk.get_data()
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0].x, 1.1)
        self.assertEqual(data[0].y, -5)
        self.assertEqual(data[0].sigma, 0.8)

class TestGzippedScript(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        f = tempfile.NamedTemporaryFile(suffix='.gz', delete=False)
        self.filename = f.name
        gf = gzip.GzipFile(fileobj=f)
        gf.write("M=1\n X[0]=1.1, Y[0]=-5, S[0]=0.8")
        gf.close()

    def tearDown(self):
        os.unlink(self.filename)

    def test_gzipped_script(self):
        self.ftk.get_ui_api().exec_fityk_script(self.filename)
        data = self.ftk.get_data()
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0].x, 1.1)
        self.assertEqual(data[0].y, -5)
        self.assertEqual(data[0].sigma, 0.8)


if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python

import os
import sys
import math
import unittest
import fityk

class TestGuessability(unittest.TestCase):
    def setUp(self):
        xx = [(n-5)/2. for n in range(25)]
        yy = [11.2/(1+(x-3.3)**2) + math.sin(40*x)/40 - x/10. for x in xx]
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        self.ftk.load_data(0, xx, yy, [1]*len(xx), "loren")
        #self.ftk.execute("info state > debug.fit")

def create_guessability_test(func_type):
    def do_test(self):
        self.ftk.execute("guess %f = " + func_type)
        f = self.ftk.get_function('f')
        self.assertIsInstance(f, fityk.Func)
        self.assertEqual(f.get_template_name(), func_type)
        try:
            ctr = f.get_param_value("Center")
            if func_type not in ['Sigmoid']:
                self.assertAlmostEqual(ctr, 3.3, delta=0.3)
        except fityk.ExecuteError:
            pass
        self.ftk.execute("delete %f")
    return do_test

def add_guessability_tests():
    all_types = fityk.Fityk().get_info('types')
    for func_type in all_types.split():
        if func_type not in ['FCJAsymm']:
            test = create_guessability_test(func_type)
            test.__name__ = 'test_' + func_type
            setattr(TestGuessability, test.__name__, test)

add_guessability_tests()

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# run tests with: python -m unittest test_tranform
#                 python -m unittest discover

import os
import sys
import unittest
import fityk
try:
    import numpy
except ImportError:
    numpy = None

def get_data_as_lists(ftk):
    data = ftk.get_data()
    return [p.x for p in data], [p.y for p in data], [p.sigma for p in data]

class TestTransform(unittest.TestCase):
    def setUp(self):
        self.x = [(n-5)/2. for n in range(20)]
        self.y = [x*(x-3.1) for x in self.x]
        self.sigma = [1 for n in self.y]
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        self.ftk.load_data(0, self.x, self.y, self.sigma, "test-data")

    def test_load(self):
        xx, yy, ss = get_data_as_lists(self.ftk)
        self.assertEqual(xx, self.x)
        self.assertEqual(yy, self.y)
        self.assertEqual(ss, self.sigma)
        #print self.x
        #print self.y

    def assert_expr(self, expr, val, places=None):
        expr_val = self.ftk.calculate_expr(expr)
        if places is None:
            self.assertEqual(expr_val, val)
        else:
            self.assertAlmostEqual(expr_val, val, places=places)

    def test_expr(self):
        xx, yy, ss = get_data_as_lists(self.ftk)
        M = len(yy)
        self.assert_expr("M", M)
        self.assert_expr("count(y>0)", len([y for y in yy if y > 0]))
        self.assert_expr("sum(x)", sum(xx))
        self.assert_expr("sum(y)", sum(yy))
        self.assert_expr("darea(y)",
                         sum(yy[n]*(xx[min(n+1, M-1)]-xx[max(n-1, 0)])/2
                             for n in range(M)))
        if not numpy:
            return
        self.assert_expr("avg(x)",    numpy.mean(xx))
        self.assert_expr("min(x)",    numpy.min(xx))
        self.assert_expr("max(x)",    numpy.max(xx))
        self.assert_expr("stddev(x)", numpy.std(xx, ddof=1))
        self.assert_expr("avg(y)",    numpy.mean(yy), places=12)
        self.assert_expr("min(y)",    numpy.min(yy))
        self.assert_expr("max(y)",    numpy.max(yy))
        self.assert_expr("stddev(y)", numpy.std(yy, ddof=1), places=12)
        self.assert_expr("argmin(y)", xx[numpy.argmin(yy)])
        self.assert_expr("argmax(y)", xx[numpy.argmax(yy)])

    def test_delete(self):
        self.ftk.execute("delete(x < 0)")
        xx, yy, ss = get_data_as_lists(self.ftk)
        self.assertEqual(xx, [x for x in self.x if x >= 0])

    def test_delete2(self):
        self.ftk.execute("A = x >= 0; delete(not a)")
        xx, yy, ss = get_data_as_lists(self.ftk)
        self.assertEqual(xx, [x for x in self.x if x >= 0])

    def test_normalize(self):
        self.ftk.execute("Y = y / darea(y)")
        self.assertEqual(self.ftk.calculate_expr("darea(y)"), 1)

    def test_set_y(self):
        self.ftk.execute("Y[3] = 1.2; Y[-2] = 12.34")
        xx, yy, ss = get_data_as_lists(self.ftk)
        self.assertEqual(yy[3], 1.2)
        self.assertEqual(yy[-2], 12.34)

    def test_xy_swap(self):
        self.ftk.execute("X=y, Y=x") # swap & sort!
        xx, yy, ss = get_data_as_lists(self.ftk)
        self.assertEqual(xx, sorted(self.y))
        xymap = { x: y for x, y in zip(self.y, self.x) }
        self.assertEqual(yy, [xymap[x] for x in xx])
        self.assertEqual(ss, self.sigma)


class TestFuncProperties(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        self.ftk.execute("%f = Quadratic(~1, 2, ~3)")
        self.ftk.execute("%g = Constant(7.5)")
        self.ftk.execute("F = %f + %g")

    def _check(self, expr, val, places=12):
        expr_val = self.ftk.calculate_expr(expr)
        self.assertAlmostEqual(expr_val, val, places=places)

    def test_numarea(self):
        # Integral of 1+2*x+3*x^2 = x+x^2+x^3| from 2 to 4 = 84-14 = 70
        self._check("%f.numarea(2, 4, 1000)", 70, places=5)
        self._check("%g.numarea(2, 4, 3)", 2*7.5)
        self._check("F.numarea(2, 4, 300)", 70 + 2*7.5, places=4)

    def test_findx(self):
        self._check("%f.findx(0, 5, 6)", 1.0, places=12)
        self._check("%f.findx(-1.5, 1.2, 6)", 1.0, places=12)
        self.ftk.calculate_expr("%g.findx(-10, 10, 7.5)") # anything
        self._check("F.findx(-1.5, 1.2, 13.5)", 1.0, places=12)
        self.ftk.execute("Z = Constant(0.2)")
        self._check("F.findx(-1.5, 1.2, 13.5)", 1.0-0.2, places=12)

    def test_extremum(self):
        self._check("%f.extremum(-10, 10)", -1.0/3, places=13)
        self.ftk.calculate_expr("%g.extremum(-10, 10)") # anything
        self._check("F.extremum(-10, 10)", -1.0/3, places=13)
        self.ftk.execute("Z = Constant(0.2)")
        self._check("F.extremum(-10, 10)", -1.0/3-0.2, places=12)

if __name__ == '__main__':
    unittest.main()




#!/usr/bin/env python
"""
Equivalent of cfityk in Python.
"""

import os
import sys
try:
    import readline
except ImportError:
    readline = None
import atexit
from optparse import OptionParser
import subprocess
import bisect
import functools
from math import isnan

import fityk

_PROMPT = "=-> "
_PROMPT2 = "... "

_gnuplot = None

def read_line():
    try:
        line = raw_input(_PROMPT)
    except EOFError:
        raise fityk.ExitRequestedException()
    while line.endswith("\\"):
        cont = raw_input(_PROMPT2)
        line = line[:-1] + cont
    return line

def get_filename_completions(text):
    head, tail = os.path.split(text)
    dirname = head or os.getcwd()
    try:
        return [os.path.join(head, f) for f in os.listdir(dirname)
                if f.startswith(tail)]
    except:
        return []

class Completer:
    def __init__(self, ftk):
        self.ftk = ftk
        self.clist = None
    def complete(self, text, state):
        if state == 0:
            self.clist = fityk.complete_fityk_line(self.ftk,
                                 readline.get_line_buffer(),
                                 readline.get_begidx(), readline.get_endidx(),
                                 text)
            if len(self.clist) == 1 and self.clist[0] == "":
                self.clist = get_filename_completions(text)
        if state < len(self.clist):
            return self.clist[state]
        else:
            return None


def main():
    config_dir = os.path.join(os.path.expanduser("~"), fityk.config_dirname())

    if readline:
        histfile = os.path.join(config_dir, "history")
        if hasattr(readline, "read_history_file"):
            try:
                readline.read_history_file(histfile)
            except IOError:
                pass
            atexit.register(readline.write_history_file, histfile)

    fityk.interrupt_computations_on_sigint()

    parser = OptionParser("Usage: %prog [-h] [-V] [-c <str>]"
                          " [script or data file...]")
    parser.add_option("-V", "--version", action="store_true",
                      help="output version information and exit")
    parser.add_option("-c", "--cmd", action="append", default=[],
                      help="script passed in as string")
    parser.add_option("-I", "--no-init", action="store_true",
                      help="don't process $HOME/.fityk/init file")
    parser.add_option("-q", "--quit", action="store_true",
                      help="don't enter interactive shell")
    (options, args) = parser.parse_args()

    f = fityk.Fityk()
    ui = f.get_ui_api()

    ui.connect_draw_plot_py(functools.partial(plot_in_gnuplot, f))

    if options.version:
        print("cfityk.py %s" % f.get_info("version").split()[-1])
        return

    if not options.no_init:
        init_file = os.path.join(config_dir, fityk.startup_commands_filename())
        if os.path.exists(init_file):
            sys.stderr.write(" -- init file: %s --\n" % init_file)
            ui.exec_fityk_script(init_file)
            sys.stderr.write(" -- end of init file --\n")

    if readline:
        readline.parse_and_bind("tab: complete")
        readline.set_completer_delims(" \t\n\"\\'`@$><=;|&{(:") # default+":"
        completer = Completer(f)
        readline.set_completer(completer.complete)

    try:
        for s in options.cmd:
            ui.exec_and_log(s)
        for arg in args:
            f.process_cmd_line_arg(arg)

        if not options.quit:
            while True:
                line = read_line()
                ui.exec_and_log(line)
                print("")
    except fityk.ExitRequestedException:
        sys.stderr.write("\nbye...\n")


def plot_in_gnuplot(f, mode):
    global _gnuplot

    # _gnuplot is False -- failed to open
    # _gnuplot is None  -- not initialized
    if _gnuplot is False:
        return

    # plot only the active dataset and model
    dm_number = f.get_default_dataset()
    points = f.get_data(dm_number)
    left_x = f.get_view_boundary('L')
    right_x = f.get_view_boundary('R')
    begin = bisect.bisect_left(points, fityk.Point(left_x,0))
    end = bisect.bisect_right(points, fityk.Point(right_x,0))
    has_points = (begin != end)

    if _gnuplot is None and has_points:
        try:
            _gnuplot = subprocess.Popen(['gnuplot'], stdin=subprocess.PIPE)
        except OSError:
            _gnuplot = False
            sys.stderr.write("WARNING: Failed to open gnuplot.\n")

    if not _gnuplot:
        return

    # send "plot ..." through the pipe to gnuplot
    plot_string = ("plot %s '-' title \"data\", '-' title \"sum\" with line\n"
                    % f.get_info("view"))
    _gnuplot.stdin.write(plot_string)

    # data
    if has_points:
        for p in points[begin:end]:
            if p.is_active and not isnan(p.x) and not isnan(p.y):
                _gnuplot.stdin.write("%f  %f\n" % (p.x, p.y))
    else:
        _gnuplot.stdin.write("0.0  0.0\n")
    _gnuplot.stdin.write("e\n") # gnuplot needs 'e' at the end of data

    # model
    if has_points:
        for p in points[begin:end]:
            if p.is_active and not isnan(p.x):
                y = f.get_model_value(p.x, dm_number)
                if not isnan(y):
                    _gnuplot.stdin.write("%f  %f\n" % (p.x, y))
    else:
        _gnuplot.stdin.write("0.0  0.0\n")
    _gnuplot.stdin.write("e\n")

if __name__ == '__main__':
    main()



#!/usr/bin/env python

# run tests with: python -m unittest test_model
#             or  python -m unittest discover

import os
import sys
import unittest
import fityk


class TestCopy(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)

    def test_copy_func(self):
        self.ftk.execute("%f = Gaussian(~10, ~0, 0.5)")
        self.ftk.execute("%g = copy(%f)")
        self.assertEqual(self.ftk.get_parameter_count(), 4)
        self.ftk.execute("%g.center = ~2.3")
        self.assertEqual(self.ftk.get_parameter_count(), 4)
        self.ftk.execute("F = %f + %g")
        self.ftk.execute("@+ = 0")
        self.ftk.execute("@1.F = copy(@0.F)")
        self.assertEqual(self.ftk.get_parameter_count(), 8)
        self.ftk.execute("@+ = 0")
        self.ftk.execute("@2.F = @0.F")
        self.assertEqual(self.ftk.get_parameter_count(), 8)
        self.ftk.execute("%g.center = ~2.4")
        d0_funcs = self.ftk.get_components(0)
        d1_funcs = self.ftk.get_components(1)
        d2_funcs = self.ftk.get_components(2)
        self.assertEqual(d0_funcs[0].get_param_value('center'), 0)
        self.assertEqual(d0_funcs[1].get_param_value('center'), 2.4)
        self.assertEqual(d1_funcs[0].get_param_value('center'), 0)
        self.assertEqual(d1_funcs[1].get_param_value('center'), 2.3)
        self.assertEqual(d2_funcs[0].get_param_value('center'), 0)
        self.assertEqual(d2_funcs[1].get_param_value('center'), 2.4)

    def test_copy_var(self):
        self.ftk.execute("%f = Gaussian(~10, ~0, 0.5)")
        self.ftk.execute("$v = copy(%f.height)")
        v = self.ftk.get_variable("v")
        self.assertEqual(v.value(), 10)

class TestBounds(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        self.ftk.execute("$a = ~3.14 [-1:4.5]")

    def test_basic(self):
        a = self.ftk.get_variable("a")
        self.assertEqual(a.value(), 3.14)
        self.assertEqual(a.domain.lo, -1)
        self.assertEqual(a.domain.hi, 4.5)
        self.assertFalse(a.domain.lo_inf())
        self.assertFalse(a.domain.hi_inf())

    def test_change_value(self):
        a = self.ftk.get_variable("a")
        self.assertRaises(fityk.SyntaxError, self.ftk.execute, "$a = ~")
        self.assertRaises(fityk.SyntaxError, self.ftk.execute, "$a = ~~9")
        self.assertRaises(fityk.SyntaxError, self.ftk.execute, "$a = ~-sin(2)")
        self.ftk.execute("$a = ~{-sin(2)}")
        self.ftk.execute("$a = ~-0.15")
        self.assertEqual(a.domain.lo, -1)
        self.assertEqual(a.domain.hi, 4.5)
        self.ftk.execute("$a = ~3.15 [:77]")
        self.assertTrue(a.domain.lo_inf())
        self.assertEqual(a.domain.hi, 77)
        self.ftk.execute("$a = ~3.15 [:]")
        self.assertTrue(a.domain.lo_inf())
        self.assertTrue(a.domain.hi_inf())

    def test_assign_var(self):
        self.ftk.execute("$b = $a") # b is compound variable
        b = self.ftk.get_variable("b")
        self.assertEqual(b.value(), 3.14)
        self.assertTrue(b.domain.lo_inf())
        self.assertTrue(b.domain.hi_inf())
        self.ftk.execute("$c = ~0.1 [0.0:0.2]")
        self.ftk.execute("$c = $a")
        c = self.ftk.get_variable("c")
        self.assertEqual(c.value(), 3.14)
        self.assertEqual(c.domain.lo, 0.0)
        self.assertEqual(c.domain.hi, 0.2)

    def test_assign_value(self):
        self.ftk.execute("$b = ~{$a}")
        b = self.ftk.get_variable("b")
        self.assertEqual(b.value(), 3.14)
        self.assertTrue(b.domain.lo_inf())
        self.assertTrue(b.domain.hi_inf())

    def test_copy_var(self):
        self.ftk.execute("$b = copy($a)")
        b = self.ftk.get_variable("b")
        self.assertEqual(b.value(), 3.14)
        self.assertEqual(b.domain.lo, -1)
        self.assertEqual(b.domain.hi, 4.5)


class TestDefaultValues(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)

    def test_pseudovoigt_shape_domain(self):
        self.ftk.execute("%f = PseudoVoigt(height=~11, center=~12, hwhm=~13)")
        f = self.ftk.get_function('f')
        shape = self.ftk.get_variable(f.var_name('shape'))
        self.assertEqual(shape.value(), 0.5)
        self.assertEqual(shape.domain.lo, 0)
        self.assertEqual(shape.domain.hi, 1)

    def test_defval_bounds(self):
        self.ftk.execute("define My(a=3, b=0.1[-1:1]) = a*exp(b*x)")
        self.ftk.execute("%f=My(a=3)")
        self.assertEqual(self.ftk.get_variable("%f.b").name, "_2")
        a = self.ftk.get_variable("_2")
        self.assertEqual(a.value(), 0.1)
        self.assertEqual(a.domain.lo, -1)
        self.assertEqual(a.domain.hi, 1)


class TestBoxConstraints(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        xx = [(n-5)/2. for n in range(25)]
        yy = [-7+3*x for x in xx]
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        self.ftk.load_data(0, xx, yy, [1]*len(xx), "line")
        self.ftk.execute("F = Linear(~0, ~0[:2.5])")
        self.methods = ["mpfit"]
        if self.ftk.get_info("compiler"):
            self.methods += ["nlopt_lbfgs", "nlopt_nm", "nlopt_bobyqa",
                             "nlopt_sbplx"]

    def fit(self, fitting_method):
        self.ftk.set_option_as_string("fitting_method", fitting_method)
        self.ftk.execute("fit")
        return self.ftk.calculate_expr("F[0].a1")

    def test_box(self):
        for m in self.methods:
            self.assertAlmostEqual(self.fit(m), 2.5)

    def test_no_box(self):
        self.ftk.set_option_as_number("box_constraints", 0)
        for m in self.methods:
            self.assertAlmostEqual(self.fit(m), 3.0)



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

# run tests with: python -m unittest test_info
#             or  python -m unittest discover

import unittest
import fityk

class TestFormula(unittest.TestCase):
    def setUp(self):
        self.ftk = fityk.Fityk()
        self.ftk.set_option_as_number("verbosity", -1)
        self.voigt = "Voigt(926, 43.2, 0.144, 0.1)"
        self.splitvoigt = "SplitVoigt(926, 43.2, 0.144, 0.143, 0.1, 0.13)"
        self.splitvoigt_formula = ('x < 43.2 ? Voigt(926, 43.2, 0.144, 0.1) '
                                            ': Voigt(926, 43.2, 0.143, 0.13)')
    def test_voigt(self):
        self.ftk.execute("F = " + self.voigt)
        formula = self.ftk.get_info("formula")
        self.assertEqual(formula, self.voigt)
        gaussian_fwhm=self.ftk.calculate_expr
        f_voigt = self.ftk.get_components(0)[0]
        gauss_fwhm = f_voigt.get_param_value('GaussianFWHM')
        self.assertEqual(round(gauss_fwhm, 13), 0.2397757280134) # not verified
    def test_voigt_s(self):
        self.ftk.execute("F = " + self.voigt)
        formula = self.ftk.get_info("simplified_formula")
        self.assertEqual(formula, self.voigt)
    def test_splitvoigt(self):
        self.ftk.execute("F = " + self.splitvoigt)
        formula = self.ftk.get_info("formula")
        self.assertEqual(formula, self.splitvoigt_formula)
    def test_splitvoigt_s(self):
        self.ftk.execute("F = " + self.splitvoigt)
        formula = self.ftk.get_info("simplified_formula")
        self.assertEqual(formula, self.splitvoigt_formula)

if __name__ == '__main__':
    unittest.main()





import re

from pygments.lexer import RegexLexer
from pygments.token import Text, Name, Comment, String, Generic
from sphinx import addnodes
from docutils import nodes

class FitykLexer(RegexLexer):
    name = 'fityklexer'

    tokens = {
            'root': [
                (r"'[^']*'", String.Single),
                (r'[#].*?$', Comment),
                (r'^=-> ', Generic.Prompt),
                (r"[^'#\n]+", Text),
                ],
            }

comment_re = re.compile(r'(\(\*.*?\*\))')

def doctree_read(app, doctree):
    env = app.builder.env
    for node in doctree.traverse(addnodes.productionlist):
        for production in node:
            if not isinstance(production, addnodes.production):
                continue
            if not isinstance(production[-1], nodes.Text):
                continue
            parts = comment_re.split(production.pop().astext())
            new_nodes = []
            for s in parts:
                if comment_re.match(s):
                    new_nodes.append(nodes.emphasis(s, s))
                elif s:
                    new_nodes.append(nodes.Text(s))
            production += new_nodes

def role_ftype(name, rawtext, text, lineno, inliner, options={}, content=[]):
    node = nodes.strong(text, text)
    assert text.count('(') in (0, 1)
    assert text.count('(') == text.count(')')
    assert ')' not in text or text.endswith(')')
    m = re.search(r'\((.*?)\)', text)
    node['ids'] = [m.group(1) if m else text]
    return [node], []

def setup(app):
    app.add_lexer('fityk', FitykLexer);
    app.connect('doctree-read', doctree_read)
    app.add_role('ftype', role_ftype)




#!/usr/bin/env python

import os.path, sys
from fityk import Fityk

class GaussianFitter(Fityk):
    def __init__(self, filename):
        Fityk.__init__(self)
        if not os.path.isfile(filename):
            raise ValueError("File `%s' not found." % filename)
        self.filename = filename
        self.execute("@0 < '%s'" % filename)
        print "Data info:", self.get_info("data", 0)

    def run(self):
        self.execute("guess Gaussian")
        gauss = self.all_functions()[-1] # the last function (just created)
        print "initial Gaussian center: %g" % gauss.get_param_value("center")
        print "Fitting %s ..." % self.filename
        self.execute("fit")
        print "WSSR=", self.get_wssr()
        print "Gaussian center: %g" % gauss.get_param_value("center")

    def save_session(self, filename):
        self.execute("info state >'%s'" % filename)

f = Fityk()
print f.get_info("version")
print "ln(2) =", f.calculate_expr("ln(2)")
del f

g = GaussianFitter(os.path.join(os.path.dirname(sys.argv[0]), "nacl01.dat"))
g.run()
g.save_session("tmp_save.fit")

